<?php
/**
 * Project: ServerSync
 * User: Smile
 * Date: 10/16/2019
 * Time: 12:57 AM
 */


class App
{
    public $db_conf_local = [];
    public $db_conf_local_master = [];

    public $db_conf_server = [];
    public $db_conf_server_master = [];

    public $db_server = null;
    public $db_local  = null;

    const SERVER_TYPE_SYNC = 'sync';
    const SERVER_TYPE_SERVER = 'server';

    /** @var object */
    public $settings;

    /**
     * App constructor.
     */
    public function __construct()
    {
        date_default_timezone_set('Asia/Kolkata');
        $this->settings = new ServerHelper\Settings();
    }

    /**
     * @throws Exception
     */
    public function connect()
    {
        $this->db_conf_local = (array) $config_ld  = $this->settings->LocalDatabase;
        $this->db_conf_local_master = (array) $this->settings->LocalMasterDatabase;

        $this->db_local = new MysqliDb([
            'host'     => $config_ld->HostName,
            'username' => $config_ld->UserName,
            'password' => $config_ld->Password,
            'db'       => $config_ld->Database,
            'port'     => 3306,
            'charset'  => 'utf8'
        ]);

        $this->db_local ->rawQuery('SET time_zone = "+05:30";');

        $this->changeServer(self::SERVER_TYPE_SYNC);
    }

    /**
     * @param $type
     * @throws Exception
     */
    public function changeServer($type)
    {
        if ($type === self::SERVER_TYPE_SYNC) {
            $this->db_conf_server_master = (array) $this->settings->SyncMasterDatabase;
            $this->db_conf_server = (array) $config_sd  = $this->settings->SyncDatabase;
        }
        else {
            $this->db_conf_server_master = (array) $this->settings->ServerMasterDatabase;
            $this->db_conf_server = (array) $config_sd  = $this->settings->ServerDatabase;
        }

        $this->db_server = new MysqliDb([
            'host'     => $config_sd->HostName,
            'username' => $config_sd->UserName,
            'password' => $config_sd->Password,
            'db'       => $config_sd->Database,
            'port'     => 3306,
            'charset'  => 'utf8'
        ]);

        $this->db_server->rawQuery('SET time_zone = "+05:30";');
    }

    public function GetStoredValue($key, $value = 1111)
    {
        return $this->settings->GetStoredValue($key, $value);
    }

    public function SetStoredValue($key, $value)
    {
        $this->settings->SetStoredValue($key, $value);
    }

    public function log($message)
    {
        echo $message . "\n";
    }

    public function logError($message)
    {
		$this->settings->Log($message, 'sync');
        echo $message . "\n";
    }

    public function logException(Exception $e)
    {
        if ($e->getCode() == 404) {
            $this->log($e->getMessage());
        }
        else {
            $this->logError($e->getMessage());
        }
    }
}
