<?php
/**
 * Project: ServerSync
 * User: Smile
 * Date: 10/15/2019
 * Time: 11:55 PM
 */


class ItemMasterPull extends PullPush
{
    protected $lock_local = [
        'tblitemmaster', 'tblitemmasterattachment', 'tblitemstockbybranch', 'tbllocbybranch', 'tblitemstockbybranchbyloc',
        'tbldefaultsettings', 'tbltaxstru', 'tbllistunit'
    ];

    protected $lock_server = ['tblitemmaster', 'tblitemmasterattachment', 'tbltaxstru', 'tbllistunit'];

    protected $sync_type = self::SYNC_TYPE_PULL;


    protected function loadItems()
    {
        $result = $this->server
            ->where('createdDate', $this->last_sync_time, '>=')
            ->where('ModifiedDate', $this->last_sync_time, '>=', 'OR')
            ->get('tblitemmaster');

        if (!$result) {
            throw new Exception('NO DATA', 404);
        }

        $items = [];
        $item_attach = [];

        $item_ids = [];
        foreach ($result as $row) {
            $id = $row['ItemId'];
            $items[$id] = $row;
            $item_attach[$id] = [];
            $item_ids[] = $id;
        }

        if (!count($item_ids)) {
            throw new Exception('NO DATA', 404);
        }

        $result = $this->server
            ->where('ItemId', $item_ids, 'IN')
            ->get('tblitemmasterattachment');

        foreach ($result as $row) {
            $item_attach[$row['ItemId']][] = $row;
        }

        $unit = $this->server
            ->map('unitname')
            ->get('tbllistunit');

        $tax = $this->server
            ->map('taxstrucode')
            ->get('tbltaxstru');

        return [
            'items' => $items,
            'attachments' => $item_attach,
            'tax' => $tax,
            'unit' => $unit
        ];
    }

    protected function saveItems($data)
    {
        foreach ($data['items'] as $id => $it) {
            $this->local
                ->onDuplicate(array_keys($it))
                ->insert('tblitemmaster', $it);


            $this->throwExceptionOnLocalError();

            $this->local
                ->where('ItemId', $id)
                ->delete('tblitemmasterattachment');

            foreach ($data['attachments'][$id] as $it_attach) {
                $it_attach['ItemId'] = $id;

                $this->local->insert('tblitemmasterattachment', $it_attach);

                $this->throwExceptionOnLocalError();
            }
        }

        if (count($data['unit'])) {
            $this->local
                ->onDuplicate(array_keys(reset($data['unit'])))
                ->insertMulti('tbllistunit', $data['unit']);

            //$this->throwExceptionOnLocalError();
        }

        if (count($data['tax'])) {
            $this->local
                ->onDuplicate(array_keys(reset($data['tax'])))
                ->insertMulti('tbltaxstru', $data['tax']);

            //$this->throwExceptionOnLocalError();
        }
    }
}
