<?php
/**
 * Project: ServerSync
 * User: Smile
 * Date: 10/16/2019
 * Time: 1:59 AM
 */


class POSPush extends PullPush
{
    protected $lock_local = ['tblpos', 'tblposdetails','tblposunsaved','tblposunsaveddetails'];

    protected $lock_server = ['tblpos', 'tblposdetails','tblposunsaved','tblposunsaveddetails'];

    protected $sync_type = self::SYNC_TYPE_PUSH;

    private $pos;
    private $pos_unsaved;

    /**
     * @throws Exception
     */
    protected function loadItems()
    {
        try {
            $this->pos = $this->loadPOS();
        } catch (Exception $e) {
            if ($e->getCode() != 404)
                throw $e;
        }

        try {
            $this->pos_unsaved = $this->loadPOSUnSaved();
        } catch (Exception $e) {
            if ($e->getCode() != 404)
                throw $e;
        }

        return [];
    }

    /**
     * @param $data
     * @throws Exception
     */
    protected function saveItems($data)
    {
        if ($this->pos)
            $this->savePOS();

        if ($this->pos_unsaved)
            $this->savePOSUnSaved();
    }

    /**
     * @return array
     * @throws Exception
     */
    private function loadPOS()
    {
        $pos = [];
        $pos_details = [];

        $result = $this->local
            ->where('IsSynced', '0')
            ->get('tblpos');

        $pos_ids = [];
        foreach ($result as $row) {
            $id = $row['posid'];
            $pos_ids[] = $id;
            unset($row['posid']);
            $pos[$id] = $row;
            $pos_details[$id] = [];
        }

        if (!count($pos_ids)) {
            $this->throwNoDataException();
        }

        $result = $this->local
            ->where('postblid', $pos_ids, 'IN')
            ->get('tblposdetails');

        foreach ($result as $row) {
            unset($row['posdtltblid']);
            $pos_details[$row['postblid']][] = $row;
        }

        return [
            'pos' => $pos,
            'pos_details' => $pos_details
        ];
    }

    /**
     * @return array
     * @throws Exception
     */
    private function loadPOSUnSaved()
    {
        $pos_unsaved = [];
        $pos_unsaved_details = [];

        $result = $this->local
            ->where('IsSynced', '0')
            ->where('isPending', '0')
            ->get('tblposunsaved');

        $pos_unsaved_ids = [];
        foreach ($result as $row) {
            $id = $row['posid'];
            $pos_unsaved_ids[] = $id;
            unset($row['posid']);
            $pos_unsaved[$id] = $row;
            $pos_unsaved_details[$id] = [];
        }

        if (!count($pos_unsaved_ids)) {
            $this->throwNoDataException();
        }

        $result = $this->local
            ->where('postblid', $pos_unsaved_ids, 'IN')
            ->get('tblposunsaveddetails');

        foreach ($result as $row) {
            unset($row['posdtltblid']);
            $pos_unsaved_details[$row['postblid']][] = $row;
        }

        return [
            'pos_unsaved' => $pos_unsaved,
            'pos_unsaved_details' => $pos_unsaved_details
        ];
    }

    /**
     * @throws Exception
     */
    private function savePOS()
    {
        foreach ($this->pos['pos'] as $id => $po) {
            $this->server
                ->insert('tblpos', $po);

            $this->throwExceptionOnServerError();

            $iid = $this->server->getInsertId();

            foreach ($this->pos['pos_details'][$id] as $pos_detail) {
                $pos_detail['postblid'] = $iid;
                $this->server
                    ->insert('tblposdetails', $pos_detail);

                $this->throwExceptionOnServerError();
            }

			$this->local
				->where('posid', $id)
				->update('tblpos', ['IsSynced' => '1']);

            $this->throwExceptionOnLocalError();
        }
    }

    /**
     * @throws Exception
     */
    private function savePOSUnSaved()
    {
        foreach ($this->pos_unsaved['pos_unsaved'] as $id => $po) {
            $this->server
                ->insert('tblposunsaved', $po);

            $this->throwExceptionOnServerError();

			$this->local
				->where('posid', $id)
				->update('tblposunsaved', ['IsSynced' => '1']);

            $this->throwExceptionOnLocalError();

            $iid = $this->server->getInsertId();

            foreach ($this->pos_unsaved['pos_unsaved_details'][$id] as $pos_unsaved_detail) {
                $pos_unsaved_detail['postblid'] = $iid;
                $this->server
                    ->insert('tblposunsaveddetails', $pos_unsaved_detail);

                $this->throwExceptionOnServerError();
            }
        }
    }
}
