<?php
/**
 * Project: ServerSync
 * User: Smile
 * Date: 10/16/2019
 * Time: 1:47 AM
 */


class POSReturnPull extends PullPush
{
    protected $lock_local = ['tblposreturn', 'tblposreturndetails'];

    protected $lock_server = ['tblposreturn', 'tblposreturndetails'];

    protected $sync_type = self::SYNC_TYPE_PULL;

    /**
     * @throws Exception
     */
    protected function loadItems()
    {
        $pos = [];
        $pos_details = [];

        $result = $this->server
            ->where('CreatedDateTime', $this->last_sync_time, '>=')
            ->get('tblposreturn');

        $pos_ids = [];
        foreach ($result as $row) {
            $id = $row['posid'];
            $pos_ids[] = $id;
            unset($row['posid']);
            $pos[$id] = $row;
            $pos_details[$id] = [];
        }

        if (!count($pos_ids)) {
            $this->throwNoDataException();
        }

        $result = $this->server
            ->where('postblid', $pos_ids, 'IN')
            ->get('tblposreturndetails');

        foreach ($result as $row) {
            unset($row['posdtltblid']);
            $pos_details[$row['postblid']][] = $row;
        }

        return [
            'pos_return' => $pos,
            'pos_return_details' => $pos_details
        ];
    }

    /**
     * @param $data
     * @throws Exception
     */
    protected function saveItems($data)
    {
        foreach ($data['pos_return'] as $id => $po) {
            $this->local
                ->insert('tblposreturn', $po);

            $this->throwExceptionOnLocalError();

            $iid = $this->local->getInsertId();

            foreach ($data['pos_return_details'][$id] as $pos_detail) {
                $pos_detail['postblid'] = $iid;
                $this->local
                    ->insert('tblposreturndetails', $pos_detail);

                $this->throwExceptionOnLocalError();
            }
        }
    }
}
